//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
//------------------------------------------------------------------------------
//
//  File:  ether.c
//
//  Core Ethernet routines for the Intel Mainstone II bootloader.
//
//------------------------------------------------------------------------------
/* 
** Copyright 2000-2003 Intel Corporation All Rights Reserved.
**
** Portions of the source code contained or described herein and all documents
** related to such source code (Material) are owned by Intel Corporation
** or its suppliers or licensors and is licensed by Microsoft Corporation for distribution.  
** Title to the Material remains with Intel Corporation or its suppliers and licensors. 
** Use of the Materials is subject to the terms of the Microsoft license agreement which accompanied the Materials.  
** No other license under any patent, copyright, trade secret or other intellectual
** property right is granted to or conferred upon you by disclosure or
** delivery of the Materials, either expressly, by implication, inducement,
** estoppel or otherwise 
** Some portion of the Materials may be copyrighted by Microsoft Corporation.
*/

#include <windows.h>
#include <bulverde.h>
#include <mainstoneii.h>
#include <ethdbg.h>
#include <ndis.h>
#include <rndismini.h>
#include <halether.h>
#include <oal_memory.h>
#include <ethdrv.h>
#include <ceddk.h>
#include "loader.h"

// Miscellaneous definitions.
//
#define CARD_DETECT             0x20
#define PCMCIA_POWER            0x0F
#define PCMCIA_RESET            0x10
#define VS1                     0x40
#define VS2                     0x80
#define PCMCIA_VCC3V_VPP0V      0x08
#define PCMCIA_VCC5V_VPP0V      0x04
#define CIS_R0                  0x3F8
#define OSCR_OFFSET             0x10        // OSCR0
#define SDBG_USB_SERIAL         101

// Function pointers to the support library functions of the currently installed debug ethernet controller.
//
PFN_EDBG_INIT                     pfnEDbgInit;
PFN_EDBG_GET_FRAME                pfnEDbgGetFrame;
PFN_EDBG_SEND_FRAME               pfnEDbgSendFrame;
PFN_EDBG_ENABLE_INTS              pfnEDbgEnableInts;
PFN_EDBG_DISABLE_INTS             pfnEDbgDisableInts;
PFN_EDBG_GET_PENDING_INTS         pfnEDbgGetPendingInts;
PFN_EDBG_READ_EEPROM              pfnEDbgReadEEPROM;
PFN_EDBG_WRITE_EEPROM             pfnEDbgWriteEEPROM;
PFN_EDBG_SET_OPTIONS              pfnEDbgSetOptions;
#ifdef IMGSHAREETH
PFN_EDBG_CURRENT_PACKET_FILTER    pfnCurrentPacketFilter;
PFN_EDBG_MULTICAST_LIST           pfnMulticastList;
#endif

extern BOOL RndisInit( BYTE *pbBaseAddress, DWORD dwMultiplier, USHORT MacAddr[3]);
extern EBOOT_CFG    g_EbootCFG;

static void Wait(UINT32 microSeconds)
{
    volatile UINT32 *TimerOSCRAddress= (volatile UINT32 *) OALPAtoVA((BULVERDE_BASE_REG_PA_OST + OSCR_OFFSET), FALSE);
    UINT32 Value, Time;

    Time   = *TimerOSCRAddress;
    Value = Time + (microSeconds * 4);
    if (Value < Time)
    {  // test for wrap.
        while (Time < *TimerOSCRAddress);
    }
    while (*TimerOSCRAddress <= Value);

}


void msWait(UINT32 msVal) 
{
    Wait(msVal*1000);
}


VOID OALStall(UINT32 microSeconds)
{
    Wait(microSeconds);
}


BOOL LocatePCMCIACard(BOOT_DEVICE_TYPE EthDevice, UINT32 SlotAddress)
{
    UINT32 SlotVoltage = PCMCIA_VCC3V_VPP0V;
    volatile UINT32 *pSRCR = NULL;
//    volatile MAINSTONEII_BLR_REGS *pBLRegs = (volatile MAINSTONEII_BLR_REGS *) OALPAtoVA(MAINSTONEII_BASE_REG_PA_FPGA, FALSE);
    volatile BULVERDE_MEMCTRL_REG *pMEMCTRLRegs = (volatile BULVERDE_MEMCTRL_REG *) OALPAtoVA(BULVERDE_BASE_REG_PA_MEMC, FALSE);

    switch (EthDevice)
    {
/*    case ETH_DEVICE_PCMCIA0:
        pSRCR = (volatile UINT32 *)&(pBLRegs->pcmcia0_srcr);
        break;
    case ETH_DEVICE_PCMCIA1:       
        pSRCR = (volatile UINT32 *)&(pBLRegs->pcmcia1_srcr);
        break;*/
    default:
        break;
    }

    if (!pSRCR)
    {
        return(FALSE);
    }

    // Initialize the PCMCIA interface if a card is detected.
    //
    if (!((*pSRCR & CARD_DETECT) == CARD_DETECT))
    {
        KITLOutputDebugString("INFO: Performing PCMCIA NIC initialization.  Please wait...\r\n");

        // Configure the MAX1602EE power supply based on the voltage requirement of the card.
        //
        if (((*pSRCR & VS1) == VS1) && ((*pSRCR & VS2) == VS2))
        {
            // 5 volt card detected.
            //
            SlotVoltage = PCMCIA_VCC5V_VPP0V;
        }
        else
        {
            // 3.3 volt card detected.
            //
            SlotVoltage = PCMCIA_VCC3V_VPP0V;
        }

        *pSRCR &= ~(PCMCIA_POWER | PCMCIA_RESET);
        msWait(50);

        *pSRCR |= SlotVoltage;
        msWait(50);

        *pSRCR |= PCMCIA_RESET;
        msWait(50);

        *pSRCR &= ~PCMCIA_RESET;
        msWait(50);

        pMEMCTRLRegs->mecr = 0x3;          
        msWait(50);         

        return (TRUE);
    }

    return (FALSE);
}


static BOOL VerifyEthernetPCMCIACard(UINT32 SlotAddress)
{
    UINT8 Attribute;
    UINT8 *pAttribute = (UINT8 *) SlotAddress;
    UINT8 *pLastAttribute;


    // End of the PCMCIA attribute list.
    //
    pLastAttribute = pAttribute + 0x200;

    //
    Attribute = *pAttribute;
    if (Attribute != (UINT8) 0x01)
    {
        KITLOutputDebugString("ERROR: Invalid PCMCIA configuration data 0x%x.\r\n", Attribute);
        return (FALSE);
    }

    // Look for tuple.
    //
    while ((Attribute != (UINT8) 0x21) && (pAttribute < pLastAttribute))
    {
        pAttribute += sizeof(UINT16);                      // Step to tuple length.
        Attribute = *pAttribute;
        pAttribute += sizeof(UINT16) + (sizeof(UINT16) * Attribute);    // Step over tuple.
        Attribute = *pAttribute;
    }

    // Find a valid tuple?
    //
    if (pAttribute < pLastAttribute)
    {
        pAttribute += sizeof(UINT16);                     // Defined tuple length.
        if ((Attribute = *pAttribute) == 2)
        {
            pAttribute += sizeof(UINT16);
            Attribute = *pAttribute;

            // Found an Ethernet card...
            //
            if (Attribute == 0x06)
            {
                return (TRUE);
            }
        }
    }

    return (FALSE);

}


// EthDevice == 0 --> Try to initialize PCMCIA Slot 0.
// EthDevice == 1 --> Try to initialize PCMCIA Slot 1.
// EthDevice == 2 --> Try to initialize DM9000.
// EthDevice == 3 --> Try to initialize RNDIS
UINT32 InitSpecifiedEthDevice(OAL_KITL_ARGS *pKITLArgs, UINT32 BootDevice)
{
    UINT32 EbootDeviceAddress = 0;
    UINT32 Offset;      
    UINT8 *pAttribute = NULL;
    UINT8 uByte;
    UINT32 SlotAddress = 0;

//    volatile MAINSTONEII_BLR_REGS *pBLRegs = (volatile MAINSTONEII_BLR_REGS *) OALPAtoVA(MAINSTONEII_BASE_REG_PA_FPGA, FALSE);

    
    // Determine the NIC's address.
    //
    switch (BootDevice)
    {
    case ETH_DEVICE_PCMCIA0: 
        SlotAddress = (UINT32) OALPAtoVA(BULVERDE_BASE_REG_PA_PCMCIA_S0_ATTR, FALSE);
        KITLOutputDebugString("INFO: Trying to locate/initialize PCMCIA NIC in slot 0...\r\n");
        break;
    case ETH_DEVICE_PCMCIA1:
        SlotAddress = (UINT32) OALPAtoVA(BULVERDE_BASE_REG_PA_PCMCIA_S1_ATTR, FALSE);
        KITLOutputDebugString("INFO: Trying to locate/initialize PCMCIA NIC in slot 1...\r\n");
        break;
    case ETH_DEVICE_USB:
        KITLOutputDebugString("INFO: Trying to initialize USB RNDIS...\r\n");
        break;
    case SER_DEVICE_USB:
        KITLOutputDebugString("INFO: Trying to initialize USB SERIAL...\r\n");
        break;
    case ETH_DEVICE_DM9000:
    default:
        SlotAddress = (UINT32) OALPAtoVA(MAINSTONEII_BASE_REG_PA_DM9000_ETHERNET, FALSE);
        KITLOutputDebugString("INFO: Trying to initialize the built-in DM9000 NIC...\r\n");
        break;
    }

    if (BootDevice == ETH_DEVICE_USB)
    {
        // Set address
        EbootDeviceAddress = (UINT32)OALPAtoVA(BULVERDE_BASE_REG_PA_UDC, FALSE);  // Base Address for USB Client

        pKITLArgs->mac[0] = g_EbootCFG.RNDISMac[0];
        pKITLArgs->mac[1] = g_EbootCFG.RNDISMac[1];
        pKITLArgs->mac[2] = g_EbootCFG.RNDISMac[2];

        // init USB registers
        if (HostMiniInit((PCHAR) NULL, 1, pKITLArgs->mac))
        {
            pfnEDbgInit           = RndisInit;
            pfnEDbgEnableInts     = RndisEnableInts;
            pfnEDbgDisableInts    = RndisDisableInts;    
            pfnEDbgGetFrame       = RndisEDbgGetFrame;       
            pfnEDbgSendFrame      = RndisEDbgSendFrame;      
            pfnEDbgReadEEPROM     = NULL;     
            pfnEDbgWriteEEPROM    = NULL;    

            // TODO: These two functons are provided in \oak\drivers\ethdbg\rne_mdd but not in \oak\drivers\netcard\rndismini

            pfnEDbgGetPendingInts = RndisGetPendingInts;  // TODO: This is called in OEMEthISR - what to do
            pfnEDbgSetOptions     = RndisSetOptions;   // TODO: This function does nothing

#ifdef IMGSHAREETH
            pfnCurrentPacketFilter = RndisCurrentPacketFilter;
            pfnMulticastList       = RndisMulticastList;
#endif

            pKITLArgs->flags |= OAL_KITL_FLAGS_POLL;

            // Save the device location information for later use.
            //
            pKITLArgs->devLoc.IfcType     = Internal;
            pKITLArgs->devLoc.BusNumber   = 0;
            pKITLArgs->devLoc.PhysicalLoc = (PVOID)BULVERDE_BASE_REG_PA_UDC;    // not a real Ethernet card
            pKITLArgs->devLoc.LogicalLoc  = (DWORD)pKITLArgs->devLoc.PhysicalLoc;

            return (EDBG_USB_RNDIS);
        }
        else
        {
            KITLOutputDebugString("ERROR: Failed to initialize Bulverde Rndis USB Ethernet controller.\n");
        }
    }
    else
    // USB SERIAL?
    //
    if (BootDevice == SER_DEVICE_USB)
    {
        // Save off the MAC address so we can at least have a device name
        pKITLArgs->mac[0] = g_EbootCFG.RNDISMac[0];
        pKITLArgs->mac[1] = g_EbootCFG.RNDISMac[1];
        pKITLArgs->mac[2] = g_EbootCFG.RNDISMac[2];

        // Save the device location information for later use.
        pKITLArgs->devLoc.IfcType     = Internal;
        pKITLArgs->devLoc.BusNumber   = 0;
        pKITLArgs->devLoc.PhysicalLoc = (PVOID)BULVERDE_BASE_REG_PA_USBH;    // Just has to be different than RNDIS
        pKITLArgs->devLoc.LogicalLoc  = (DWORD)pKITLArgs->devLoc.PhysicalLoc;


        return (SDBG_USB_SERIAL);
    }
    else
    // PCMCIA NIC?
    //
    if ((BootDevice == ETH_DEVICE_PCMCIA0) || (BootDevice == ETH_DEVICE_PCMCIA1))
    {
        // Locate the NIC card in the specified slot.
        //
        if (LocatePCMCIACard(BootDevice, SlotAddress))
        {
            // Determine the PCMCIA card type.
            //
            if (VerifyEthernetPCMCIACard(SlotAddress))
            {
/*                if ((((pBLRegs->pcmcia0_srcr & PCMCIA_POWER) == PCMCIA_VCC5V_VPP0V) && (BootDevice == ETH_DEVICE_PCMCIA0)) ||
                    (((pBLRegs->pcmcia1_srcr & PCMCIA_POWER) == PCMCIA_VCC5V_VPP0V) && (BootDevice == ETH_DEVICE_PCMCIA1)))
                {
                    KITLOutputDebugString("INFO: 5.0V Card detected in slot %d.\r\n", ((BootDevice == ETH_DEVICE_PCMCIA0)?0:1));

                    uByte       = 0x21;
                    Offset      = CIS_R0;
                    pAttribute  = (UINT8 *) SlotAddress;
                    pAttribute += Offset; 
                    *pAttribute = uByte;

                    uByte       = 0x1;            
                    Offset      = CIS_R0 + 2;
                    pAttribute  = (UINT8 *) SlotAddress;
                    pAttribute += Offset; 
                    *pAttribute = uByte;
                }
                else*/
                {
                    KITLOutputDebugString("INFO: 3.3V Card detected in slot %d.\r\n", ((BootDevice == ETH_DEVICE_PCMCIA0)?0:1));

                    uByte       = 0x20;           
                    Offset      = CIS_R0;
                    pAttribute  = (UINT8 *) SlotAddress;
                    pAttribute += Offset; 
                    *pAttribute = uByte;
                }

                EbootDeviceAddress = (BootDevice == ETH_DEVICE_PCMCIA0) ? (UINT32) OALPAtoVA(BULVERDE_BASE_REG_PA_PCMCIA_S0_IO, FALSE) + 0x300 : \
                                                                         (UINT32) OALPAtoVA(BULVERDE_BASE_REG_PA_PCMCIA_S1_IO, FALSE) + 0x300;

                // Only supported PCMCIA NIC is an NE2000-compatible NIC.  Initialize it.
                //
                if (NE2000Init((BYTE *) EbootDeviceAddress, 1, pKITLArgs->mac))
                {
                    pfnEDbgInit            = NE2000Init;
                    pfnEDbgGetFrame        = NE2000GetFrame;       
                    pfnEDbgSendFrame       = NE2000SendFrame;      
                    pfnEDbgEnableInts      = NE2000EnableInts;     
                    pfnEDbgDisableInts     = NE2000DisableInts;    
#ifdef IMGSHAREETH
                    pfnCurrentPacketFilter = NULL;
                    pfnMulticastList       = NULL;
#endif

                    // Save the device location information for later use.
                    //
                    pKITLArgs->devLoc.IfcType     = Internal;
                    pKITLArgs->devLoc.BusNumber   = 0;
                    pKITLArgs->devLoc.PhysicalLoc = (PVOID)((BootDevice == ETH_DEVICE_PCMCIA0) ? (BULVERDE_BASE_REG_PA_PCMCIA_S0_IO + 0x300) : (BULVERDE_BASE_REG_PA_PCMCIA_S1_IO + 0x300));
                    pKITLArgs->devLoc.LogicalLoc  = (DWORD)pKITLArgs->devLoc.PhysicalLoc;

                    KITLOutputDebugString("INFO: NE2000 Ethernet controller initialized.\n");
                    return(EDBG_ADAPTER_NE2000);
                }
                else
                {
                    KITLOutputDebugString("ERROR: Failed to initialize NE2000 Ethernet controller.\n");
                }
            }
        }

        return (-1);
    }
    else
    {
        // Use the DM9000 NIC.
        //
        EbootDeviceAddress = (SlotAddress + 0x300);

        if (DM9000Init((UINT8 *) EbootDeviceAddress, 0, pKITLArgs->mac))
        {
            pfnEDbgInit            = (PFN_EDBG_INIT)        DM9000Init;
            pfnEDbgGetFrame        = (PFN_EDBG_GET_FRAME)   DM9000GetFrame;       
            pfnEDbgSendFrame       = (PFN_EDBG_SEND_FRAME)  DM9000SendFrame;      
            pfnEDbgEnableInts      = (PFN_EDBG_ENABLE_INTS) DM9000EnableInts;     
            pfnEDbgDisableInts     = (PFN_EDBG_DISABLE_INTS)DM9000DisableInts;    
#ifdef IMGSHAREETH
            pfnCurrentPacketFilter = (PFN_EDBG_CURRENT_PACKET_FILTER) DM9000CurrentPacketFilter;
            pfnMulticastList       = (PFN_EDBG_MULTICAST_LIST) DM9000MulticastList;
#endif

            // Save the device location information for later use.
            //
            pKITLArgs->devLoc.IfcType     = Internal;
            pKITLArgs->devLoc.BusNumber   = 0;
            pKITLArgs->devLoc.PhysicalLoc = (PVOID)(MAINSTONEII_BASE_REG_PA_DM9000_ETHERNET + 0x300);
            pKITLArgs->devLoc.LogicalLoc  = (DWORD)pKITLArgs->devLoc.PhysicalLoc;

            KITLOutputDebugString("INFO: DM9000 Ethernet controller initialized.\r\n");
            return(EDBG_ADAPTER_SMC9000);
        }
        else
        {
            KITLOutputDebugString("ERROR: Failed to initialize DM9000 Ethernet controller.\n");
        }
    }

    return (-1);

}


BOOL OEMEthGetFrame(BYTE *pData,       // OUT - Receives frame data
                    UINT16 *pwLength)  // IN  - Length of Rx buffer
{
    return (pfnEDbgGetFrame(pData, pwLength));
}


BOOL OEMEthSendFrame(BYTE *pData,     // IN - Data buffer
                     DWORD dwLength)  // IN - Length of buffer
{
    int retries = 0;

    // Let's be persistant here
    while (retries++ < 4)
    {
        if (!pfnEDbgSendFrame(pData, dwLength))
            return (TRUE);
        KITLOutputDebugString("!OEMEthSendFrame failure, retry %u\n",retries);
    }
    return (FALSE);
}
