//
// Copyright (c) Microsoft Corporation.  All rights reserved.
//
//
// Use of this sample source code is subject to the terms of the Microsoft
// license agreement under which you licensed this sample source code. If
// you did not accept the terms of the license agreement, you are not
// authorized to use this sample source code. For the terms of the license,
// please see the license agreement between you and Microsoft or, if applicable,
// see the LICENSE.RTF on your install media or the root of your tools installation.
// THE SAMPLE SOURCE CODE IS PROVIDED "AS IS", WITH NO WARRANTIES.
//
//------------------------------------------------------------------------------
//
//  File: args.h
//
//  This file defines the shared argument structure used by the bootloader and
//  the OS image.
//
#ifndef _ARGS_H_
#define _ARGS_H_

//------------------------------------------------------------------------------

#define BSP_ARGS_VERSION    3

#define BSP_ARGS_QUERY_DBGSERIAL    BSP_ARGS_QUERY      // Query debug serial port.
#define BSP_ARGS_QUERY_HIVECLEAN    BSP_ARGS_QUERY+1    // Query hive clean flag.
#define BSP_ARGS_QUERY_CLEANBOOT    BSP_ARGS_QUERY+2    // Query clean boot flag.
#define BSP_ARGS_QUERY_FORMATPART   BSP_ARGS_QUERY+3    // Query format partition flag.
#define BSP_ARGS_QUERY_SIGNEDSTATE  BSP_ARGS_QUERY+4    // Query image signed state.

typedef struct {
    OAL_ARGS_HEADER header;
    UINT8 deviceId[16];                 // Device identification
    OAL_KITL_ARGS kitl;
    UINT32 dbgSerPhysAddr;              // Debug serial physical address
    UINT8 uuid[16];
    BOOL bUpdateMode;                   // TRUE = Enter update mode on reboot.
    BOOL bHiveCleanFlag;                // TRUE = Clean hive at boot
    BOOL bCleanBootFlag;                // TRUE = Clear RAM, hive, user store at boot
    BOOL bFormatPartFlag;               // TRUE = Format partion when mounted at boot
    UINT32 ImageSignedState;            // enum {IMAGE_NOT_SIGNED, IMAGE_TEST_SIGNED, IMAGE_FINAL_SIGNED}
} BSP_ARGS, *PBSP_ARGS;


// Called with physical address in IPL
// Called with virtual address in EBOOT
VOID OALArgsInit(BSP_ARGS *pBSPArgs);

#endif // _ARGS_H_.
